"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var react_text_mask_1 = require("react-text-mask");
var di_1 = require("../../../di");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var configurations_definitions_interface_1 = require("../../../configurations-definitions.interface");
var field_1 = require("../field");
var progress_1 = require("../../progress");
var keyboard_1 = require("../../keyboard");
var env_1 = require("../../../env");
var BaseTextField = /** @class */ (function (_super) {
    __extends(BaseTextField, _super);
    /**
     * @stable [25.02.2019]
     * @param {TProps} props
     */
    function BaseTextField(props) {
        var _this = _super.call(this, props) || this;
        _this.defaultActions = [];
        _this.mirrorInputRef = React.createRef();
        _this.keyboardRef = React.createRef();
        _this.onKeyboardChange = _this.onKeyboardChange.bind(_this);
        _this.onKeyboardWindowMouseDownHandler = _this.onKeyboardWindowMouseDownHandler.bind(_this);
        util_1.ifNotFalseThanValue(props.clearActionRendered, function () { return _this.addClearAction(); });
        return _this;
    }
    /**
     * @stable [25.02.2019]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.render = function () {
        return (React.createElement("div", { className: this.getFieldClassName() },
            this.props.children,
            this.getSelfElement(),
            this.getMessageElement(),
            this.getErrorMessageElement(),
            this.getAttachmentElement(),
            this.getKeyboardElement()));
    };
    /**
     * @stable [25.02.2019]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getInputElement = function () {
        return R.isNil(this.getFieldMask())
            ? _super.prototype.getInputElement.call(this)
            : this.getMaskedInputElement();
    };
    /**
     * @stable [01.06.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getAttachmentElement = function () {
        return null;
    };
    /**
     * @stable [07.11.2018]
     * @returns {string}
     */
    BaseTextField.prototype.getSelfElementClassName = function () {
        return util_1.toClassName(_super.prototype.getSelfElementClassName.call(this), 'rac-text-field', 'rac-flex', 'rac-flex-row');
    };
    BaseTextField.prototype.addClearAction = function () {
        var this0 = this;
        var clearAction = {
            type: 'close',
            onClick: function () {
                this0.clearValue();
            },
            get disabled() {
                return this0.isFieldInactive() || !this0.isValuePresent();
            },
        };
        this.defaultActions.push(clearAction);
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.keyboardElement = function () {
        return (React.createElement(keyboard_1.Keyboard, __assign({ ref: this.keyboardRef, field: this.input, onClose: this.closeSyntheticKeyboard, onChange: this.onKeyboardChange }, this.getKeyboardConfiguration())));
    };
    /**
     * @stable [13.01.2019]
     * @returns {boolean}
     */
    BaseTextField.prototype.isKeyboardOpened = function () {
        return _super.prototype.isKeyboardOpened.call(this) || this.getKeyboardConfiguration().renderToBody === false;
    };
    /**
     * @stable [21.11.2018]
     * @returns {IKeyboardConfiguration}
     */
    BaseTextField.prototype.getKeyboardConfiguration = function () {
        return this.props.keyboardConfiguration || {};
    };
    /**
     * @stable [23.02.2019]
     */
    BaseTextField.prototype.openSyntheticKeyboard = function () {
        var result = _super.prototype.openSyntheticKeyboard.call(this);
        if (result) {
            this.keyboardWindowMouseDownSubscriber =
                this.eventManager.subscribe(window, 'mousedown', this.onKeyboardWindowMouseDownHandler);
        }
        return result;
    };
    /**
     * @stable [23.02.2019]
     */
    BaseTextField.prototype.onCloseKeyboard = function () {
        _super.prototype.onCloseKeyboard.call(this);
        if (util_1.isFn(this.keyboardWindowMouseDownSubscriber)) {
            this.keyboardWindowMouseDownSubscriber();
            this.keyboardWindowMouseDownSubscriber = null;
        }
    };
    /**
     * @stable [04.09.2018]
     */
    BaseTextField.prototype.refreshCaretPosition = function () {
        _super.prototype.refreshCaretPosition.call(this);
        this.input.scrollLeft = this.input.scrollWidth;
    };
    /**
     * @stable [04.09.2018]
     * @returns {number}
     */
    BaseTextField.prototype.getCaretPosition = function () {
        return this.isValuePresent()
            ? Math.min(this.jMirrorInput.width(), this.jqInput.width())
            : 0;
    };
    /**
     * @stable [16.02.2019]
     * @param {IBasicEvent} e
     */
    BaseTextField.prototype.onKeyboardWindowMouseDownHandler = function (e) {
        var clickedEl = e.target;
        if (this.domAccessor.hasParent('.rac-action-close-icon', clickedEl)) {
            if (this.isValuePresent()) {
                this.clearValue();
            }
        }
        else if (!(this.input === clickedEl
            || this.domAccessor.hasElements(clickedEl, this.keyboardRef.current.getSelf()))) {
            this.closeSyntheticKeyboard();
        }
    };
    Object.defineProperty(BaseTextField.prototype, "actions", {
        get: function () {
            var props = this.props;
            if (props.actionsPosition === configurations_definitions_interface_1.FieldActionPositionEnum.LEFT) {
                return (this.defaultActions || []).concat(props.actions || []);
            }
            else {
                return (this.props.actions || []).concat(this.defaultActions || []);
            }
        },
        enumerable: true,
        configurable: true
    });
    BaseTextField.prototype.getSelfElement = function () {
        var _this = this;
        var props = this.props;
        return util_1.orNull(props.fieldRendered !== false, function () { return (React.createElement("div", { ref: 'self', style: props.style, className: _this.getSelfElementClassName() },
            _this.getPrefixLabelElement(),
            React.createElement("div", { className: _this.getInputElementWrapperClassName() },
                _this.getInputElement(),
                _this.getLabelElement(),
                _this.getMirrorInputElement(),
                _this.getInputCaretElement(),
                _this.getInputAttachmentElement()),
            util_1.orNull(_this.actions, _this.actions.map(function (action) { return _this.uiFactory.makeIcon({
                type: action.type,
                title: action.title && _this.t(action.title),
                className: action.className,
                disabled: R.isNil(action.disabled)
                    ? _this.isFieldInactive()
                    : action.disabled,
                onClick: function (event) {
                    if (action.onClick) {
                        util_1.cancelEvent(event);
                        action.onClick(event);
                    }
                },
            }); })),
            _this.getProgressLabelElement())); });
    };
    /**
     *
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getProgressLabelElement = function () {
        return util_1.orNull(this.inProgress(), function () { return React.createElement(progress_1.ProgressLabel, { className: 'rac-field-loader rac-absolute-center-position' }); });
    };
    /**
     * @stable [15.09.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getPrefixLabelElement = function () {
        var props = this.props;
        return (util_1.orNull(props.prefixLabel, React.createElement("span", { className: 'rac-field-prefix-label rac-absolute-left-center-position' }, props.prefixLabel)));
    };
    /**
     * @stable [12.09.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getMirrorInputElement = function () {
        var props = this.props;
        if (!props.useKeyboard || !this.isValuePresent() || !this.useSyntheticCursor) {
            return null;
        }
        // TODO Move to support
        var value = this.value;
        var content = String((props.type === 'password'
            ? value.replace(/./g, env_1.ENV.passwordInputPlaceholder)
            : value)).replace(/ /g, definitions_interface_1.UNI_CODES.noBreakSpace);
        return (React.createElement("span", { ref: this.mirrorInputRef, className: util_1.toClassName(this.getInputElementProps().className, 'rac-field-input-mirror', 'rac-invisible') }, content));
    };
    /**
     * @stable [03.10.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getLabelElement = function () {
        var _this = this;
        var props = this.props;
        return util_1.ifNotNilThanValue(props.label, function () { return (React.createElement("label", { className: 'rac-field-label' }, _this.t(props.label))); });
    };
    /**
     * @stable [04.09.2018]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getInputCaretElement = function () {
        var _this = this;
        var state = this.state;
        var textOffset = 2;
        return util_1.orNull(this.useSyntheticCursor && state.keyboardOpened && state.caretVisibility && !R.isNil(state.caretPosition), function () { return (React.createElement("div", { className: 'rac-field-input-caret', style: { left: state.caretPosition + util_1.parseValueAtPx(_this.jqInput.css('paddingLeft')) - textOffset } }, "|")); });
    };
    /**
     * @stable [04.09.2018]
     * @param {string} rawValue
     */
    BaseTextField.prototype.onKeyboardChange = function (rawValue) {
        // Synthetic event creation to reduce source code
        var syntheticEvent = { target: { value: rawValue } };
        // To handle various field types, NumberField, etc..
        this.onChangeManually(this.getRawValueFromEvent(syntheticEvent));
    };
    Object.defineProperty(BaseTextField.prototype, "jMirrorInput", {
        /**
         * TODO domAccessor
         */
        get: function () {
            return util_1.toJqEl(this.mirrorInputRef.current);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [25.02.2019]
     * @returns {JSX.Element}
     */
    BaseTextField.prototype.getMaskedInputElement = function () {
        var mask = this.getFieldMask();
        var props = this.props;
        return (React.createElement(react_text_mask_1.default, __assign({ guide: util_1.nvl(props.maskGuide, BaseTextField.DEFAULT_MASK_GUIDE), placeholderChar: util_1.nvl(props.maskPlaceholderChar, definitions_interface_1.UNI_CODES.space), mask: mask }, this.getInputElementProps())));
    };
    BaseTextField.DEFAULT_MASK_GUIDE = false;
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.EventManager),
        __metadata("design:type", Object)
    ], BaseTextField.prototype, "eventManager", void 0);
    return BaseTextField;
}(field_1.Field));
exports.BaseTextField = BaseTextField;
//# sourceMappingURL=base-textfield.component.js.map