"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var definitions_interface_1 = require("../../definitions.interface");
var util_1 = require("../../util");
var field_1 = require("../field");
var list_1 = require("../list");
var base_1 = require("../base");
var layout_1 = require("../layout");
var overlay_1 = require("../overlay");
var Menu = /** @class */ (function (_super) {
    __extends(Menu, _super);
    /**
     * @stable [31.07.2018]
     * @param {IMenuProps} props
     */
    function Menu(props) {
        var _this = _super.call(this, props) || this;
        _this.fieldRef = React.createRef();
        _this.menuAnchorRef = React.createRef();
        _this.hide = _this.hide.bind(_this);
        _this.onSelect = _this.onSelect.bind(_this);
        _this.onInputChange = _this.onInputChange.bind(_this);
        _this.onCloseAction = _this.onCloseAction.bind(_this);
        _this.state = { opened: false };
        return _this;
    }
    /**
     * @stable [07.06.2018]
     * @returns {JSX.Element}
     */
    Menu.prototype.render = function () {
        var _this = this;
        var props = this.props;
        var state = this.state;
        return (React.createElement("div", { ref: this.menuAnchorRef, className: this.uiFactory.menuAnchor },
            React.createElement("div", { ref: this.getSelfRef(), style: { width: util_1.calc(props.width) }, className: util_1.toClassName('rac-menu', props.className, this.uiFactory.menu, this.uiFactory.menuSurface, props.renderToCenterOfBody && 'rac-absolute-center-position rac-menu-centered') },
                util_1.orNull(props.renderToCenterOfBody && this.isOpen(), function () { return React.createElement(overlay_1.Overlay, { onClick: _this.onCloseAction }); }),
                util_1.orNull(props.renderToCenterOfBody, function () { return _this.uiFactory.makeIcon({ type: 'close', className: 'rac-menu-close-action', onClick: _this.onCloseAction }); }),
                util_1.orNull(props.useFilter, function () { return (React.createElement(layout_1.FlexLayout, { full: false, row: true, className: 'rac-menu-field-wrapper' },
                    React.createElement(field_1.TextField, { ref: _this.fieldRef, value: state.filter, placeholder: props.filterPlaceholder || _this.settings.messages.filterPlaceholderMessage, errorMessageRendered: false, onChange: _this.onInputChange }))); }),
                this.listElement)));
    };
    /**
     * @stable [04.10.2018]
     */
    Menu.prototype.show = function () {
        var _this = this;
        var props = this.props;
        this.setState({ filter: definitions_interface_1.UNDEF, opened: true }, function () { return props.useFilter && _this.field.setFocus(); });
        if (!R.isNil(props.renderToX) || !R.isNil(props.renderToY)) {
            util_1.setAbsoluteOffsetByCoordinates(this.menuAnchorRef.current, props.renderToX, props.renderToY);
        }
    };
    /**
     * @stable [29.11.2018]
     */
    Menu.prototype.hide = function () {
        this.setState({ opened: false });
    };
    /**
     * @stable [29.11.2018]
     * @returns {boolean}
     */
    Menu.prototype.isOpen = function () {
        return this.state.opened;
    };
    Object.defineProperty(Menu.prototype, "menuItems", {
        /**
         * @stable [06.08.2018]
         * @returns {IMenuItemEntity[]}
         */
        get: function () {
            var props = this.props;
            var state = this.state;
            var query = state.filter;
            return props.useFilter
                ? props.options.filter(function (option) { return props.filter(query, option); })
                : props.options;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [17.10.2018]
     * @param {IBasicEvent} event
     */
    Menu.prototype.onCloseAction = function (event) {
        util_1.cancelEvent(event);
        this.hide();
    };
    /**
     * @stable [04.10.2018]
     * @param {string} filter
     */
    Menu.prototype.onInputChange = function (filter) {
        this.setState({ filter: filter });
    };
    Object.defineProperty(Menu.prototype, "field", {
        /**
         * @stable [07.06.2018]
         * @returns {IField}
         */
        get: function () {
            return this.fieldRef.current;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [06.08.2018]
     * @param {IBasicEvent<HTMLElement>} event
     * @param {IMenuItemEntity} option
     */
    Menu.prototype.onSelect = function (event, option) {
        this.stopEvent(event);
        var props = this.props;
        if (props.onSelect) {
            props.onSelect(option);
        }
        if (!props.multi
            || this.menuItems.length === 1 // Because a "Flux Cycle", to prevent empty list after updating
        ) {
            this.hide();
        }
    };
    Object.defineProperty(Menu.prototype, "listElement", {
        /**
         * @stable [11.12.2018]
         * @returns {JSX.Element}
         */
        get: function () {
            var _this = this;
            var props = this.props;
            var menuItems = this.menuItems;
            return (React.createElement(list_1.SimpleList, { className: this.uiFactory.list }, util_1.subArray(menuItems, props.maxCount).map(function (option) { return (React.createElement("li", { key: "menu-item-key-" + option.value, className: 'rac-list-item rac-flex', "aria-disabled": option.disabled === true, onClick: function (event) { return _this.onSelect(event, option); } }, _this.getMenuItemElement(option))); })));
        },
        enumerable: true,
        configurable: true
    });
    Menu.prototype.getMenuItemElement = function (option) {
        var _this = this;
        var props = this.props;
        var optionValueFn = function (itm) { return (itm.label ? _this.t(itm.label) : itm.value); };
        return (React.createElement(layout_1.FlexLayout, { justifyContentCenter: true, className: 'rac-menu-item' }, util_1.isFn(props.renderer)
            ? props.renderer(option)
            : (option.icon
                ? (React.createElement(layout_1.FlexLayout, { row: true, alignItemsCenter: true },
                    this.uiFactory.makeIcon({ type: option.icon, className: 'rac-menu-item-icon' }),
                    optionValueFn(option)))
                : util_1.isFn(props.tpl) ? props.tpl(option) : optionValueFn(option))));
    };
    Menu.defaultProps = {
        filter: function (query, entity) { return util_1.queryFilter(query, entity.label || entity.value); },
    };
    return Menu;
}(base_1.BaseComponent));
exports.Menu = Menu;
//# sourceMappingURL=menu.component.js.map