"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../util");
var base_1 = require("../base");
var layout_1 = require("../layout");
var dialog_1 = require("../dialog");
var viewer_1 = require("../viewer");
var Viewer = /** @class */ (function (_super) {
    __extends(Viewer, _super);
    /**
     * @stable [08.07.2018]
     * @param {TProps} props
     */
    function Viewer(props) {
        var _this = _super.call(this, props) || this;
        _this.onPreview = _this.onPreview.bind(_this);
        _this.onDialogClose = _this.onDialogClose.bind(_this);
        _this.state = {};
        return _this;
    }
    /**
     * @stable [08.07.2018]
     * @param {TProps} props
     * @param {TState} state
     */
    Viewer.prototype.componentDidUpdate = function (props, state) {
        _super.prototype.componentDidUpdate.call(this, props, state);
        var currentProps = this.props;
        if (!R.equals(currentProps.src, props.src)) {
            this.refresh();
        }
    };
    /**
     * @stable [08.07.2018]
     */
    Viewer.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        this.refresh();
    };
    /**
     * @stable [08.07.2018]
     * @returns {JSX.Element}
     */
    Viewer.prototype.render = function () {
        var _this = this;
        var state = this.state;
        var props = this.props;
        var isErrorExist = !R.isNil(state.error);
        var isSrcAbsent = R.isNil(props.src);
        var isDefaultSrcAbsent = R.isNil(props.defaultScr);
        var isSrcAndDefaultSrcAbsent = isSrcAbsent && isDefaultSrcAbsent;
        var isProgressMessageShown = this.isProgressMessageShown;
        var canShowPreview = props.usePreview && !isSrcAbsent && !isProgressMessageShown && !isSrcAndDefaultSrcAbsent && !isErrorExist;
        return (React.createElement(layout_1.FlexLayout, { style: props.style, full: props.full, className: this.getClassName() },
            isErrorExist || isSrcAndDefaultSrcAbsent
                ? (isErrorExist
                    ? (React.createElement(layout_1.CenterLayout, null, this.t(this.settings.messages.fileLoadErrorMessage)))
                    : React.createElement(viewer_1.PictureViewer, { usePreview: false }))
                : (isProgressMessageShown ? this.settings.messages.waitingMessage : this.getContentElement()),
            util_1.orNull(canShowPreview, function () { return (_this.uiFactory.makeIcon({
                type: 'search_plus',
                className: 'rac-preview-action-icon rac-absolute-center-position',
                onClick: _this.onPreview,
            })); }),
            util_1.orNull(canShowPreview, function () { return (React.createElement(dialog_1.Dialog, { ref: 'dialog', className: 'rac-preview-dialog', title: _this.settings.messages.previewMessage, closeMessage: _this.settings.messages.closeMessage, acceptable: false, onClose: _this.onDialogClose }, util_1.orNull(state.opened, function () { return _this.gePreviewElement(); }))); })));
    };
    /**
     * @stable [10.01.2019]
     * @returns {string}
     */
    Viewer.prototype.getClassName = function () {
        return util_1.toClassName('rac-viewer', this.props.className);
    };
    /**
     * @stable [08.07.2018]
     */
    Viewer.prototype.refresh = function () {
        // Do nothing
    };
    Object.defineProperty(Viewer.prototype, "isProgressMessageShown", {
        /**
         * @stable [08.07.2018]
         * @returns {boolean}
         */
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [08.07.2018]
     */
    Viewer.prototype.onDialogClose = function () {
        this.setState({ opened: false });
    };
    /**
     * @stable [08.07.2018]
     */
    Viewer.prototype.onPreview = function () {
        this.setState({ opened: true });
        var dialog = this.refs.dialog;
        dialog.activate();
    };
    return Viewer;
}(base_1.BaseComponent));
exports.Viewer = Viewer;
//# sourceMappingURL=viewer.component.js.map