"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var URI = require("urijs");
var definitions_interface_1 = require("../definitions.interface");
function buildRoute(path, params) {
    (path.match(/\:[a-zA-Z0-9]+/g) || []).forEach(function (placeholder) {
        path = path.replace(placeholder, params[placeholder.substring(1, placeholder.length)]);
    });
    return path;
}
exports.buildRoute = buildRoute;
/**
 * @stable [01.12.2018]
 * @param {string} path
 * @returns {string}
 */
exports.buildNormalizedPath = function (path) { return URI('/').segment(new URI(path).segment()).toString() || '/'; };
/**
 * @stable [21.08.2018]
 * @param {string} path
 * @param {TEntity} entity
 * @param {IKeyValue} params
 * @returns {string}
 */
exports.buildEntityRoute = function (path, entity, params) {
    return buildRoute(path, __assign({ id: entity ? entity.id : definitions_interface_1.NEW_OPTION }, params));
};
/**
 * @stable [23.10.2018]
 * @param {string} path
 * @param {IKeyValue} routes
 * @returns {{route: string; bindings: IKeyValue}}
 */
exports.findUrlPattern = function (path, routes) {
    if (path === void 0) { path = ''; }
    var bindings = {};
    var currentUriPathParts = new URI(path).path().split('/');
    var currentRoute = null;
    R.forEachObjIndexed(function (value, routePattern) {
        var currentRouteUriParts = new URI(routePattern).path().split('/');
        if (currentRouteUriParts.length === currentUriPathParts.length) {
            currentRouteUriParts.forEach(function (currentRouteUriPart, index) {
                var currentUriPathPart = currentUriPathParts[index];
                if (currentRouteUriPart.startsWith(':')) {
                    bindings[currentRouteUriPart] = currentUriPathPart;
                }
                else if (currentRouteUriPart !== currentUriPathPart) {
                    currentRoute = false;
                }
            });
            if (currentRoute !== false && R.isNil(currentRoute)) {
                currentRoute = routePattern;
            }
        }
    }, routes);
    return {
        route: currentRoute,
        bindings: bindings,
    };
};
//# sourceMappingURL=route.js.map